@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -24,6 +24,11 @@
 #include "ev-module.h"
 #include "ev-backends-manager.h"
 
+#ifdef _MACH_O_
+#include <mach-o/dyld.h>
+#endif
+
+
 static GList *ev_backends_list = NULL;
 
 typedef struct _EvBackendInfo EvBackendInfo;
@@ -56,7 +61,31 @@ backends_dir (void)
 						"backends", NULL);
 		g_free (dir);
 #else
-		backendsdir = g_strdup (EV_BACKENDSDIR);
+/*
+  #ifdef _MACH_O_
+      {char path[1024];
+       guint size = sizeof(path);
+       gchar * bindir = (gchar*)g_malloc(size);
+       if (_NSGetExecutablePath(bindir, &size) == 0){
+         backendsdir = g_build_filename (bindir, "..", "..", "lib", "evince",
+                                                EV_BACKENDSBINARYVERSION,
+                                                "backends", NULL);
+		g_print("using evince modules path == %s\n", backendsdir);
+       }
+       else
+         g_critical("Cannot get evince modules path\n");
+      }
+  #endif
+  #else
+*/
+      if (getenv("EVINCE_MODULE_PATH") != NULL){
+	 backendsdir = g_build_filename (getenv("EVINCE_MODULE_PATH"), NULL);
+	 printf("\nSetting EVINCE_MODULE_PATH == %s\n", backendsdir);
+      }
+      else {
+	 backendsdir = g_strdup (EV_BACKENDSDIR);
+      }
+  //#endif
 #endif
 	}
 
